package com.sztzjy.resource_center.controller.new_module.admin;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageInfo;
import com.google.common.reflect.TypeToken;
import com.nimbusds.jose.shaded.gson.Gson;
import com.nimbusds.jose.shaded.gson.JsonSyntaxException;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.entity.admin.*;
import com.sztzjy.resource_center.mapper.AdminComponentCodeMapper;
import com.sztzjy.resource_center.mapper.AdminJupyterTokenMapper;
import com.sztzjy.resource_center.mapper.admin.AdminFileMapper;
import com.sztzjy.resource_center.util.PageUtil;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * @author tz
 * @date 2024/7/26 10:16
 */
@RestController
@Api(tags = "管理员端组件代码案例管理")
@RequestMapping("api/admin/componentCode")
public class AdminComponentCodeController {

    @Autowired
    private IFileUtil fileUtil;
    @Value("${file.path}")
    private String filePath;
    @Resource
    AdminFileMapper adminFileMapper;
    @Resource
    AdminComponentCodeMapper adminComponentCodeMapper;

    @Autowired
    private AdminJupyterTokenMapper adminJupyterTokenMapper;

    @Value("${tch.serverAddress}")
    String JUPYTER_URL;

    @PostMapping("/add")
    @ApiOperation("新增")
    @AnonymousAccess
    @Transactional
    public ResultEntity<String> add(@RequestBody UpdateAdminDataDTO dto) {

        if (StringUtils.isBlank(dto.getStepName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入步骤名称！");
        }
        if (StringUtils.isBlank(dto.getCaseName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入案例名称！");
        }
        if (StringUtils.isBlank(dto.getCodeContent())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入代码内容！");
        }
//        if (StringUtils.isBlank(dto.getParameterContent())) {
//            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入参数释义！");
//        }


        //查询是否已有该案例名称
        AdminComponentCodeExample codeExample=new AdminComponentCodeExample();
        codeExample.createCriteria().andChapterNameEqualTo(dto.getCaseName()).andCourseNameEqualTo(dto.getStepName());
        List<AdminComponentCode> adminComponentCodes = adminComponentCodeMapper.selectByExample(codeExample);

        if(!adminComponentCodes.isEmpty()){
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "该案例已存在！");
        }

        //新增
        AdminComponentCodeWithBLOBs adminComponentCode = new AdminComponentCodeWithBLOBs();

        adminComponentCode.setCourseName(dto.getStepName());
        adminComponentCode.setChapterName(dto.getCaseName());
        adminComponentCode.setItem(dto.getCodeContent());
        adminComponentCode.setDefinition(dto.getParameterContent());
        adminComponentCode.setStatus(0); //默认下架，发布后上架
        adminComponentCode.setCreateTime(new Date());


        //新增案例
        adminComponentCodeMapper.insert(adminComponentCode);
        return new ResultEntity<>(HttpStatus.OK, "新增成功！");
    }



    @PostMapping("updateAdminData")
    @ApiOperation("编辑")
    @AnonymousAccess
    @Transactional
    public ResultEntity<String> updateAdminData(@RequestBody UpdateAdminDataDTO dto) {
        //编辑
        try {
            AdminComponentCodeWithBLOBs adminComponentCode = new AdminComponentCodeWithBLOBs();
            adminComponentCode.setId(dto.getId());
            adminComponentCode.setCourseName(dto.getStepName());
            adminComponentCode.setChapterName(dto.getCaseName());
            adminComponentCode.setItem(dto.getCodeContent());
            adminComponentCode.setDefinition(dto.getParameterContent());
            adminComponentCode.setStatus(0); //默认下架，发布后上架
            adminComponentCode.setCreateTime(new Date()); //修改时间

            // 更新对象
            adminComponentCodeMapper.updateByPrimaryKeyWithBLOBs(adminComponentCode);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "编辑失败请联系管理员！");
        }
        return new ResultEntity<>(HttpStatus.OK, "编辑成功！");
    }




    @PostMapping("deleteAdminData")
    @ApiOperation("删除")
    @AnonymousAccess
    @Transactional
    public ResultEntity<String> deleteAdminData(Integer id) {
        //删除
        try {

            // 删除对象
            adminComponentCodeMapper.deleteByPrimaryKey(id);

        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "删除失败请联系管理员！");
        }
        return new ResultEntity<>(HttpStatus.OK, "删除成功！");
    }






    @PostMapping("getAdminData")
    @ApiOperation("展示")
    @AnonymousAccess
    @Transactional
    public ResultEntity getAdminData(@RequestParam Integer index,
                                     @RequestParam Integer size,
                                     @RequestParam(required = false) String caseName ) {

        AdminComponentCodeExample codeExample=new AdminComponentCodeExample();
        if(!caseName.isEmpty()){
            codeExample.createCriteria().andChapterNameLike("%"+caseName+"%");
        }

        codeExample.setOrderByClause("create_time DESC");

        List<AdminComponentCodeWithBLOBs> adminComponentCodeWithBLOBs = adminComponentCodeMapper.selectByExampleWithBLOBs(codeExample);

        PageInfo pageInfo = PageUtil.pageHelper(adminComponentCodeWithBLOBs, index, size);


        return new ResultEntity<>(HttpStatus.OK, "成功！",pageInfo);
    }

    @PostMapping("addFile")
    @ApiOperation("编辑/文件单独新增")
    @AnonymousAccess
    private ResultEntity<HttpStatus> addFile(@RequestParam String dataId,
                                             @RequestParam MultipartFile dataFile) {
        try {
            AdminFile adminFile = new AdminFile();
            String url = fileUtil.upload(dataFile);
            adminFile.setFileId(IdUtil.randomUUID());
            adminFile.setDataCaseId(dataId);
            adminFile.setName(dataFile.getOriginalFilename());
            adminFile.setSource("管理员");
            adminFile.setFileUrl(url);
            adminFileMapper.insert(adminFile);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "上传失败，请联系管理员！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "上传成功！");
    }

    @PostMapping("deleteFile")
    @ApiOperation("编辑/单个数据文件删除")
    @AnonymousAccess
    private ResultEntity<HttpStatus> deleteFile(@RequestParam String fileId) {
        try {
            AdminFile adminFile = adminFileMapper.selectByPrimaryKey(fileId);
            if (adminFile != null) {
                adminFileMapper.deleteByPrimaryKey(adminFile.getFileId());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "删除失败，请联系管理员！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "删除成功！");
    }


    @PostMapping("updateStatus")
    @ApiOperation("发布或禁用")
    @AnonymousAccess
    private ResultEntity<String> updateStatus(@RequestParam Integer id,
                                              @ApiParam("传0禁用，传1发布上架") @RequestParam int status) {
        try {
            AdminComponentCodeWithBLOBs adminComponentCodeWithBLOBs = adminComponentCodeMapper.selectByPrimaryKey(id);
            adminComponentCodeWithBLOBs.setStatus(status);
            adminComponentCodeMapper.updateByPrimaryKeyWithBLOBs(adminComponentCodeWithBLOBs);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "发生错误，请联系管理员！");
        }
        if (status == 0) {
            return new ResultEntity<>(HttpStatus.OK, "成功禁用！");
        }
        if (status == 1) {
            return new ResultEntity<>(HttpStatus.OK, "成功发布！");
        }
        return null;
    }



    @ApiOperation("后端根据名字创建notebook")
    @GetMapping("/createNoteBook")
    @AnonymousAccess
    public ResultEntity createNoteBook(@ApiParam("参数为：案例名") String name,
                                       @ApiParam("代码内容") String codeContent) throws IOException {

        //判断有无这个案例
        String path = "/usr/local/tianzeProject/jupyter/tch/";

        // 创建表示该目录的 File 对象
        File directory = new File(path);

        // 获取目录中的所有文件和子目录
        File[] filesList = directory.listFiles();


        if (filesList != null) {
            // 使用流查找目标文件
            Optional<File> targetFile = Arrays.stream(filesList)
                    .filter(file -> file.getName().equals(name))
                    .findFirst();

            if (targetFile.isPresent()) {
                // 找到目标文件
                File file = targetFile.get();
                // 删除文件
                boolean deleted = file.delete();
                if (deleted) {
                    System.out.println("文件已删除: " + file.getAbsolutePath());
                } else {
                    System.out.println("删除文件失败: " + file.getAbsolutePath());
                }
            }
        }

        //没有就创建
        AdminJupyterTokenExample tokenExample = new AdminJupyterTokenExample();
        tokenExample.createCriteria().andTypeEqualTo(1);

        List<AdminJupyterToken> adminJupyterTokenList = adminJupyterTokenMapper.selectByExample(tokenExample);
        if (adminJupyterTokenList.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "Token不存在！");
        }

        try (CloseableHttpClient client = HttpClients.createDefault()) {
//            String notebookContent = "{"
//                    + "\"cells\": ["
//                    + "{"
//                    + "\"cell_type\": \"code\","
//                    + "\"execution_count\": null,"
//                    + "\"metadata\": {},"
//                    + "\"outputs\": [],"
//                    + "\"source\": [" + codeContent + "]"
//                    + "}"
//                    + "],"
//                    + "\"metadata\": {"
//                    + " \"kernelspec\": {"
//                    + " \"display_name\": \"Python 3\","
//                    + " \"language\": \"python\","
//                    + " \"name\": \"python3\""
//                    + " }"
//                    + "},"
//                    + "\"nbformat\": 4,"
//                    + "\"nbformat_minor\": 2"
//                    + "}";


            if (codeContent.startsWith("```python")) {
                codeContent = codeContent.replaceFirst("```python", "");
            }
            if (codeContent.endsWith("```")) {
                codeContent = codeContent.substring(0, codeContent.length() - 3);
            }

            String notebookContent = "{"
                    + "\"cells\": ["
                    + "{"
                    + "\"cell_type\": \"code\","
                    + "\"execution_count\": null,"
                    + "\"metadata\": {},"
                    + "\"outputs\": [],"
                    + "\"source\": [\"" + codeContent.replace("\n", "\\n").replace("\"", "\\\"") + "\"]"
                    + "}"
//                    + "{"
//                    + "\"cell_type\": \"raw\","
//                    + "\"execution_count\": null,"
//                    + "\"metadata\": {},"
//                    + "\"outputs\": [],"
//                    + "\"source\": [\"print('This is text!')\"]"
//                    + "},"
//                    + "{"
//                    + "\"cell_type\": \"markdown\","
//                    + "\"metadata\": {},"
//                    + "\"source\": [\"# Markdown Cell\\n\", \"This is a markdown cell.\"]"
//                    + "}"
                    + "],"
                    + "\"metadata\": {"
                    + "    \"kernelspec\": {"
                    + "        \"display_name\": \"Python 3\","
                    + "        \"language\": \"python\","
                    + "        \"name\": \"python3\""
                    + "    }"
                    + "},"
                    + "\"nbformat\": 4,"
                    + "\"nbformat_minor\": 2"
                    + "}";


            // 在根目录下创建新的笔记本
            HttpPut createNotebookRequest = new HttpPut(JUPYTER_URL + "/api/contents/" + name + ".ipynb");
            createNotebookRequest.addHeader("Authorization", "token " + adminJupyterTokenList.get(0).getToken());
            createNotebookRequest.addHeader("Content-Type", "application/json charset=UTF-8");
//createNotebookRequest.setEntity(new StringEntity("{\"type\": \"notebook\", \"content\": " + notebookContent + "}"));
            createNotebookRequest.setEntity(new StringEntity("{\"type\": \"notebook\", \"content\": " + notebookContent + "}", ContentType.APPLICATION_JSON.withCharset("UTF-8")));

            try (CloseableHttpResponse createNotebookResponse = client.execute(createNotebookRequest)) {
                int statusCode = createNotebookResponse.getStatusLine().getStatusCode();
                if (statusCode == 201) {
                    System.out.println("Notebook created successfully!");


                    String toPath = "/usr/local/tianzeProject/jupyter/stu/" + name + ".ipynb";

                    String[] commandMove = {"cp", "-f", path + name + ".ipynb", toPath};


                    Process process = Runtime.getRuntime().exec(commandMove);


                    // 等待进程执行完成
                    int exitCodeInfo = process.waitFor();
                    System.out.println(exitCodeInfo);
                    if (exitCodeInfo == 0) {
                        return new ResultEntity<>(HttpStatus.OK, "创建成功", true);
                    } else {
                        return new ResultEntity<>(HttpStatus.BAD_REQUEST, "创建失败！请联系管理员", false);
                    }
                } else {
                    String responseBody = EntityUtils.toString(createNotebookResponse.getEntity());
                    System.out.println("Failed to create notebook: " + responseBody);
                    return new ResultEntity<>(HttpStatus.OK, "已经存在，不用重复创建");
                }
            } catch (IOException e) {
                throw new RuntimeException(e);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }


        }
    }
}
